import { Injectable } from '@angular/core';

import { BehaviorSubject } from 'rxjs/BehaviorSubject';
import { Observable } from 'rxjs/Observable';

import { SongAPI } from '../core/song-api.service';

@Injectable()
export class SongStore {

  private _songs = new BehaviorSubject<Song[]>([]);

  readonly songs$ = this._songs.asObservable();

  constructor(private songAPI: SongAPI) {
    this.loadInitialData();
  }

  loadInitialData() {
    this.songAPI.getAllSongs()
      .subscribe((rawSongs: any[]) => {
        const _songs = rawSongs.map(rawSong => new Song(rawSong.artist, rawSong.title, rawSong.id));
        this._songs.next(_songs);
      }
    );
  }

  addSong(newSong: Song) {
    this.songAPI.saveSong(newSong)
      .subscribe((rawSong: any) => {
        const song = new Song(rawSong.artist, rawSong.title, rawSong.id);
        this._songs.next([...this._songs.getValue(), song]);
      }
    );
  }

  removeSong(song: Song) {
    this.songAPI.deleteSong(song)
      .subscribe((res: any) => {
        const songs = this._songs.getValue();
        songs.splice(songs.indexOf(song), 1);
        this._songs.next(songs);
      }
    );
  }
}

class Song {
  constructor (
    public artist,
    public title,
    public id?,
  ) { }
}
